package annotator;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StringReader;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;

import weka.core.stemmers.IteratedLovinsStemmer;

import com.wcohen.ss.TFIDF;
import com.wcohen.ss.api.StringWrapper;
import com.wcohen.ss.api.StringWrapperIterator;

import deeper.entities.graph.Edge;
import deeper.entities.graph.Graph;
import deeper.entities.graph.Node;
import deeper.operators.treesvm.KernelTree;
import deeper.parser.Parser;
import deeper.parser.StanfordParser;

public class Operations
{
	public static final double TOP_K_PERCENT = 0.03;
	

	/**
	 * @param args
	 */
	public static void main(String[] args)
	{
		//buildMissingDependencyTrees();
		//findSimilarDepTrees(16);
		tfidf(2417, true);
		
		//int sentenceIds[] = {1,14,66,90,164,198,205,366,575,684};
		//setSentencePriorities(sentenceIds);
	}
	
	public static void setSentencePriorities(int[] ids)
	{
		HashMap<Integer, Integer> modes = new HashMap<Integer, Integer>();
		
		for (int i=0; i<ids.length; i++)
		{
			int id = ids[i];
			
			ArrayList<SentenceComparable> sentences = tfidf(id, false);
			int max = (int)(TOP_K_PERCENT * sentences.size());
			for (int scount=0; scount<max; scount++)
			{
				SentenceComparable s = sentences.get(scount);
				if (!modes.containsKey(s.index))
					modes.put(s.index, 0);
				modes.put(s.index, modes.get(s.index)+1);
			}
		}
		
		
		for (Integer sentenceId : modes.keySet())
			System.out.println(sentenceId + "\t" + modes.get(sentenceId));
	}
	
	
	public static ArrayList<SentenceComparable> tfidf(int sentenceId, boolean print)
	{
		TFIDF tfidf = new TFIDF();
		
		StringWrapper misclassifiedSentence = null;
		ArrayList<SentenceComparable> sentences = new ArrayList<SentenceComparable>(); 
		ArrayList<StringWrapper> wrappers = new ArrayList<StringWrapper>();
		
		for (SentenceComparable sc : getPreprocessedSentences(true, true, true))
		{
			StringWrapper sw = tfidf.prepare(sc.text);
			if (sc.index == sentenceId)
				misclassifiedSentence = sw;
			else
			{
				sentences.add(sc);
				wrappers.add(sw);
			}
		}
		
		if (misclassifiedSentence == null)
		{
			System.err.println("Could not find the misclassified sentence!");
			System.exit(1);
		}
		System.out.println("Misclassified Sentence: " + misclassifiedSentence.toString());
		
		StringWrapperIterator2 it = new StringWrapperIterator2(wrappers);
		tfidf.train(it);
		
		for (int i=0; i<sentences.size(); i++)
		{
			sentences.get(i).similarity = tfidf.score(misclassifiedSentence, wrappers.get(i)); 
		}
		
		Collections.sort(sentences);
		
		if (print)
		{
	//		for (int i=0; i<Math.min(50, sentences.size()); i++)
			for (int i=0; i<sentences.size(); i++)
			{
				SentenceComparable sc = sentences.get(i);
				System.out.println(sc.index + "\t" + sc.similarity + "\t" + sc.text);
			}
	//		
			System.out.println("");
			System.out.println("Total Sentences: " + sentences.size());
			
	//		for (int i=Math.max(sentences.size()-50, 0); i<sentences.size(); i++)
	//		{
	//			SentenceComparable sc = sentences.get(i);
	//			System.out.println(sc.index + "\t" + sc.similarity + "\t" + sc.text);
	//		}
		}
		
		return sentences;
	}
	
	public static void findSimilarDepTrees(int sentenceId)
	{
		SentenceComparable misclassifiedSentence = null;
		ArrayList<SentenceComparable> sentences = new ArrayList<SentenceComparable>(); 
		
		try
		{
			DB db = new DB();
			db.connect();
			
			
			ResultSet rs = db.querySelect("SELECT s.id, s.sentence, s.dependency_tree FROM sentences as s");
			while (rs.next())
			{
				SentenceComparable sc = new SentenceComparable(null, rs.getInt("id"), rs.getString("sentence"));
				Graph dt = Graph.deserialize(new BufferedReader(new StringReader(rs.getString("dependency_tree"))));
				sc.setDepTree(dt);
				if (sc.index == sentenceId)
					misclassifiedSentence = sc;
				else
					sentences.add(sc);
			}
			
			
			db.disconnect();
		}
		catch (SQLException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		
		if (misclassifiedSentence == null)
			System.err.println("Could not find the misclassified sentence!");
		
		for (SentenceComparable sc : sentences)
		{
			sc.similarity = Cm(misclassifiedSentence.getDepTree().getRoot(), sc.getDepTree().getRoot(), misclassifiedSentence.getDepTree(), sc.getDepTree());
		}
		
		Collections.sort(sentences);
		
		for (SentenceComparable sc : sentences)
			if (sc.similarity > 0)
				System.out.println(sc.index + "\t" + sc.similarity + "\t" + sc.text);
		
	}
	
	public static String stem(String sentence)
	{
		IteratedLovinsStemmer stemmer = new IteratedLovinsStemmer();
		String[] words = sentence.split(" ");
		String result = "";
		
		for (int i=0; i<words.length; i++)
			result += stemmer.stem(words[i]) + " ";
		
		return result;
	}
	
	public static Collection<SentenceComparable> getPreprocessedSentences(boolean removePunctuation, boolean doStemming, boolean doStopwords) 
	{
		//String[] stopwords = {"a", "about", "an", "and", "any", "are", "as", "at", "be", "by", "com", "for", "from", "have", "how", "in", "is", "it", "may", "of", "on", "or", "show", "that", "the", "there", "their", "this", "to", "we", "was", "what", "when", "where", "who", "will", "with"};
		String stopwordsRegex = " a | about | an | and | any | are | as | at | be | by | com | for | from | have | how | in | is | it | may | of | on | or | show | that | the | there | their | this | to | we | was | what | when | where | who | will | with ";
		
		HashMap<Integer, Integer> entityCheck = null;
		HashMap<Integer, SentenceComparable> sentences = new HashMap<Integer, SentenceComparable>();
		int entityCount = 0;
		String sentence = "";
		
		try
		{	
			DB db = new DB();
			db.connect();
			int lastSentence = 0;
			ResultSet rs = db.querySelect("SELECT s.id, s.sentence, p.p1_offset, p.p1_length, p.p1_id, e1.name AS p1_name, e2.name AS p2_name, p.p2_offset, p.p2_length, p.p2_id" +
					"						FROM sentences AS s" +
					"						JOIN annotation_sentence_priorities as sp ON s.id=sp.sentence_id" +
					"						JOIN interaction_predictions AS p ON p.sentence_id=s.id" +
					"						JOIN entity_names AS e1 ON p.p1_id=e1.entity_id AND e1.preferred=1" +
					"						JOIN entity_names AS e2 ON p.p2_id=e2.entity_id AND e2.preferred=1" +
					"" + //						WHERE s.dependency_tree=''
					"						ORDER BY p.sentence_id, p.id" +
					"						");
			while (rs.next())
			{
				int sentenceId = rs.getInt("id"); 
				if (sentenceId != lastSentence)
				{
					entityCheck = new HashMap<Integer, Integer>();
					lastSentence = sentenceId;
					entityCount = 0;
					sentence = rs.getString("sentence");
//					String[] temp = sentence.split(" ");
//					if (temp.length > 100)
//					{
//						sentence = "";
//						for (int i=0; i<90; i++)
//							sentence += temp[i] + " ";
//					}
				}
				
				int p1id = rs.getInt("p1_id");
				int p1offset = rs.getInt("p1_offset");
				int p1length = rs.getInt("p1_length");
				int p2id = rs.getInt("p2_id");
				int p2offset = rs.getInt("p2_offset");
				int p2length = rs.getInt("p2_length");
				
				if (!entityCheck.containsKey(p1id))
				{
					String entityName = "ENTITY" + entityCount;
					sentence = sentence.replace(rs.getString("p1_name"), entityName);
					entityCount++;
					entityCheck.put(p1id, 1);
				}
				
				if (!entityCheck.containsKey(p2id))
				{
					String entityName = "ENTITY" + entityCount;
					sentence = sentence.replace(rs.getString("p2_name"), entityName);
					entityCount++;
					entityCheck.put(p2id, 1);
				}
				
				sentences.put(sentenceId, new SentenceComparable(null, sentenceId, sentence));
			}
			
			
			db.disconnect();
			
			for (SentenceComparable sc : sentences.values())
			{
				if (removePunctuation)
					sc.text = sc.text.replaceAll("[^\\w\\s]", "");
				
				if (doStopwords)
					sc.text = sc.text.replaceAll(stopwordsRegex, " ");
				
				if (doStemming)
					sc.text = stem(sc.text);
				
				sc.text = sc.text.replaceAll("\\s{2,}", " ");
			}
		}
		catch (Exception e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return sentences.values();
	}
	
	
	public static void buildMissingDependencyTrees() 
	{
		Parser parser = new StanfordParser();
		parser.init("-maxLength", "100");
		
		try
		{
			parser.parse("This is a test of the Stanford Parser.");
			
			Graph sentence1 = parser.getDependencyTree();
//			System.out.println(parser.getDependencyTree().serialize());
//			System.out.println(parser.getParseTree().serialize());
			
			//parser.parse("This is a test for the Stanford Parser.");
			HashMap<Integer, Integer> entityCheck = null;
			HashMap<Integer, String> sentences = new HashMap<Integer, String>();
			int entityCount = 0;
			String sentence = "";
			
			DB db = new DB();
			db.connect();
			int lastSentence = 0;
			ResultSet rs = db.querySelect("SELECT s.id, s.sentence, p.p1_offset, p.p1_length, p.p1_id, e1.name AS p1_name, e2.name AS p2_name, p.p2_offset, p.p2_length, p.p2_id" +
					"						FROM sentences AS s" +
					"						JOIN interaction_predictions AS p ON p.sentence_id=s.id" +
					"						JOIN entity_names AS e1 ON p.p1_id=e1.entity_id AND e1.preferred=1" +
					"						JOIN entity_names AS e2 ON p.p2_id=e2.entity_id AND e2.preferred=1" +
					"						WHERE s.dependency_tree=''" +
					"						ORDER BY p.sentence_id, p.id" +
					"						");
			while (rs.next())
			{
				int sentenceId = rs.getInt("id"); 
				if (sentenceId != lastSentence)
				{
					entityCheck = new HashMap<Integer, Integer>();
					lastSentence = sentenceId;
					entityCount = 0;
					sentence = rs.getString("sentence");
					String[] temp = sentence.split(" ");
					if (temp.length > 100)
					{
						sentence = "";
						for (int i=0; i<90; i++)
							sentence += temp[i] + " ";
					}
				}
				
				int p1id = rs.getInt("p1_id");
				int p1offset = rs.getInt("p1_offset");
				int p1length = rs.getInt("p1_length");
				int p2id = rs.getInt("p2_id");
				int p2offset = rs.getInt("p2_offset");
				int p2length = rs.getInt("p2_length");
				
				if (!entityCheck.containsKey(p1id))
				{
					String entityName = "ENTITY" + entityCount;
					sentence = sentence.replace(rs.getString("p1_name"), entityName);
					entityCount++;
					entityCheck.put(p1id, 1);
				}
				
				if (!entityCheck.containsKey(p2id))
				{
					String entityName = "ENTITY" + entityCount;
					sentence = sentence.replace(rs.getString("p2_name"), entityName);
					entityCount++;
					entityCheck.put(p2id, 1);
				}
				
				sentences.put(sentenceId, sentence);
			}
			
			ArrayList<String> statements = new ArrayList<String>();
			int count = 0;
			for (Integer sId : sentences.keySet())
			{
				String s = sentences.get(sId);
				//System.out.println(s);
				parser.parse(s);
				String parseTree = parser.getParseTree().serialize();
				String dependencyTree = parser.getDependencyTree().serialize();
				
				statements.add(String.format("UPDATE sentences SET parse_tree='%s', dependency_tree='%s' WHERE id=%d", DB.clean(parseTree), DB.clean(dependencyTree), sId));
				
				System.out.println("Processed sentence " + sId);
				
				if (count % 10 == 0 && count > 0)
				{
					db.queryBatch(statements);
					statements.clear();
					System.out.println("> Wrote to db.");
				}
				
				count++;
			}
			
			if (statements.size() > 0)
				db.queryBatch(statements);
			
			db.disconnect();
			
			//Graph sentence2 = parser.getDependencyTree();


			
			
			//double similarity = Cm(sentence1.getRoot(), sentence2.getRoot(), sentence1, sentence2);
			//System.out.println(similarity);
			
		}
		catch (Exception e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	private static double Cm(Node v1, Node v2, Graph g1, Graph g2) {
        double result = 1;
        if ((v1.word==null && v2.word==null) || 
        		((v1.word!=null && v2.word!=null) && v1.word.equals(v2.word))) {
            Collection<Edge> out1 = g1.getOutEdges(v1);
            Collection<Edge> out2 = g2.getOutEdges(v2);
            if (out1.size() > 0 && out2.size() > 0) {
                for (Edge e1 : out1) {
                    for (Edge e2 : out2) {
                        if ((e1.label==null && e2.label==null) || 
                        		((e1.label!=null && e2.label!=null) && e1.label.equals(e2.label))) {
                            //TODO: lambda parameter is not taken into account
                            result *= (Cm(g1.getDest(e1), g2.getDest(e2), g1, g2) + 2);
                        }
                    }
                }
            }
        }
        result--;
        return result;
    }

}
