/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package deeper.operators;

import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.operator.text.io.filereader.FileReader;
import com.rapidminer.operator.text.io.filereader.FileReaders;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeBoolean;
import com.rapidminer.parameter.ParameterTypeCategory;
import com.rapidminer.parameter.ParameterTypeDirectory;
import com.rapidminer.parameter.ParameterTypeList;
import com.rapidminer.parameter.ParameterTypeStringCategory;
import com.rapidminer.parameter.UndefinedParameterError;
import com.rapidminer.tools.LogService;

import deeper.entities.Interaction;
import deeper.entities.Sentence;
import deeper.entities.graph.Graph;
import deeper.entities.rm.io.AbstractIO;
import deeper.parser.Parser;
import deeper.parser.StanfordParser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileFilter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringReader;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;

/**
 * 
 * @author tfairuzo
 */
public class ParserApplier extends Operator {

	public static final String PARAMETER_PARSER = "parser";
	public static final String PARAMETER_PARSES = "parse_directories";
	// public static final String PARAMETER_SAVE_DIR = "save_directory";
	public static final String PARAMETER_PARSE_DIR = "directory";
	public static final String PARAMETER_SAVE_PARSE = "save_parse";
	public static final String PARAMETER_DATASET_NAME = "dataset_name";
	public static final String[] PARSER_LIST = new String[] { "stanford parser" };
	public static final String[] DATASET_LIST = new String[] { "aimed", "bioinfer", "hprd50", "iepa", "lll", "pubmed" };

	private InputPort textInput = getInputPorts().createPort("text", AbstractIO.class);
	private OutputPort textOutput = getOutputPorts().createPort("text");

	private Parser parser = null;

	public ParserApplier(OperatorDescription description) {
		super(description);
	}

	@Override
	public void doWork() throws OperatorException {
		getParser();
		AbstractIO abstIO = textInput.getData();
		// InteractionsIOIO result = new InteractionsIOIO();
		boolean save = getParameterAsBoolean(PARAMETER_SAVE_PARSE);

		File saveDir = null;
		// build a file lookup table
		Map<String, File> nameFileMap = new HashMap<String, File>();
		for (String[] pair : getParameterList(PARAMETER_PARSES)) {
			if (pair[0].equalsIgnoreCase(abstIO.getAbstract().corpus)) {
				saveDir = getProcess().resolveFileName(pair[1]);
				if (!saveDir.exists()) {
					saveDir.mkdirs();
				} else {
					final String pattern = abstIO.getAbstract().corpus + "." + abstIO.getAbstract().getExt_id();
					for (File file : saveDir.listFiles(new FileFilter() {

						@Override
						public boolean accept(File pathname) {
							return pathname.getName().startsWith(pattern);
						}
					})) {

						nameFileMap.put(file.getName(), file);
					}
				}
			}
		}

		// dir associated with a corpus to which the sentence belongs
		// File saveDir = nameFileMap.get(abstIO.getAbstract().corpus);
		// saveDir.mkdirs();

		for (Sentence s : abstIO.getAbstract().getSentences()) {
			if(s.text.length()==0){//skip empty sentences (should they occur)?
				LogService.getRoot().log(Level.WARNING, "Sentence " + s.getExt_id()+" is empty");
				continue;
			}
			if (s.getDepTree() == null || s.getParseTree() == null) {
				File file = nameFileMap.get(s.getExt_id());
				if (file != null) {
					FileReader reader = FileReaders.createFileReader(file, this);
					try {
						String text = reader.readFile(file, false, FileReaders.getEncoding(this));
						String[] trees = text.split("<treeseparator>");
						s.setDepTree(Graph.deserialize(new BufferedReader(new StringReader(trees[0]))));
						s.setParseTree(Graph.deserialize(new BufferedReader(new StringReader(trees[1]))));
						/*for(Interaction inter : s.getInteractions()){
							
			                System.out.println(inter.getUniqueId()+" "+inter.getProtein1()+" "+inter.getProtein2());
						}*/
					} catch (IOException ex) {
						LogService.getRoot().log(Level.SEVERE, "Can't read the file: " + file.getAbsolutePath(), ex);
					}
				} else {
					try {
						long start = System.currentTimeMillis();
						parser.parse(s.text_mod);
						s.parsingTime = System.currentTimeMillis() - start;
						// Update sentence parsing trees will trigger
						// updates of all corresponding interaction shortest
						// paths
						s.setDepTree(parser.getDependencyTree());
						s.setParseTree(parser.getParseTree());
						if (save) {
							FileWriter writer = new FileWriter(saveDir + File.separator + s.getExt_id());
							writer.write(s.getDepTree().serialize());
							writer.write("<treeseparator>");
							writer.write(s.getParseTree().serialize());
							writer.close();
						}
					} catch (Exception ex) {
						// throw new
						// OperatorException("Error while parsing the sentence: "
						// + s.text_mod, ex);
						LogService.getRoot().log(Level.WARNING, "Error while parsing the sentence: " + s.text_mod, ex);
					}
				}
			}
		}
		textOutput.deliver(abstIO);
	}

	// different parsers can be added there
	private Parser getParser() throws UndefinedParameterError {
		// TODO: not sure if this condition is needed, depend on whether the
		// Operator
		// instance is destroyed after a process execution. If not, it can save
		// several seconds for each execution
		if (parser != null) {
			return parser;
		}
		if (getParameterAsString(PARAMETER_PARSER).equals("stanford parser")) {
			parser = new StanfordParser();
			// max sentence length
			parser.init("-maxLength", "100");
		}
		return parser;
	}

	@Override
	public List<ParameterType> getParameterTypes() {
		List<ParameterType> types = super.getParameterTypes();

		ParameterType type;

		type = new ParameterTypeCategory(PARAMETER_PARSER, "Parsers that can be used", PARSER_LIST, 0);
		types.add(type);

		type = new ParameterTypeList(PARAMETER_PARSES, "In this list arbitrary directories can be specified.", new ParameterTypeStringCategory(PARAMETER_DATASET_NAME,
				"The name of dataset presented in the directory", DATASET_LIST), new ParameterTypeDirectory(PARAMETER_PARSE_DIR, "Specifies the directory with parse files (used to save as well).",
				false));
		type.setExpert(false);
		types.add(type);

		type = new ParameterTypeBoolean(PARAMETER_SAVE_PARSE, "Save parsing results", true);
		types.add(type);

		// type = new ParameterTypeDirectory(PARAMETER_SAVE_DIR,
		// "Specifies the directory to save parse results.", true);
		// type.registerDependencyCondition(new BooleanParameterCondition(this,
		// PARAMETER_SAVE_PARSE, true, true));
		// types.add(type);

		types.addAll(FileReaders.getParameterTypes(this));

		return types;
	}
}
