package deeper.operators;

import java.util.HashMap;
import java.util.List;
import java.util.logging.Level;

import abner.Tagger;

import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.tools.LogService;

import deeper.entities.Protein;
import deeper.entities.Sentence;
import deeper.entities.rm.io.AbstractIO;

public class ProteinRecognizerApplier extends Operator {

	 private InputPort textInput = getInputPorts().createPort("text", AbstractIO.class);
	 private OutputPort textOutput = getOutputPorts().createPort("text");

	
	 //TODO generic NER interface
	 private Tagger tagger;
	
	public ProteinRecognizerApplier(OperatorDescription description) {
		super(description);
	}
	
	 @Override
	 public void doWork() throws OperatorException {
		 AbstractIO abstIO = textInput.getData();
		 if(tagger==null){
			 tagger = new Tagger(Tagger.BIOCREATIVE);
			 tagger.setTokenization(false);
		 }
		 
		 for (Sentence s : abstIO.getAbstract().getSentences()) {
			 if(s.getProteins().size()==0){
				 String taggedText = tagger.tagSGML(s.text);
				 extractProteins(taggedText, s);
				 s.buildInteractions(new HashMap<List<Protein>,String>(), false, "");
				 s.rewrite();
			 }
		 }
		 textOutput.deliver(abstIO);
	 }
	
    //String in the Sentence object may be different from one passed as argument!
    //(number of spaces may be different).
    private void extractProteins(String taggedSentence, Sentence origSentence) {
        String tagged = taggedSentence;
        //String origText = tagged.replace("<PROTEIN> ", "").replace("</PROTEIN> ", "").replace("\\(", "-LBR-").replace("\\)", "-RBR-").replace("%", "percent");
        String origText = origSentence.text;
        //String findCharOff = tagged.replace("<PROTEIN> ", "").replace("</PROTEIN> ", "");
        int entID = 0;
        int findIndex = -1;
        while (tagged.contains("<PROTEIN>")) {
            //entityCount ++;
            int tagIndex = tagged.indexOf("<PROTEIN>");
            int entIndex = tagged.indexOf(" ", tagIndex);
            int entEnd = tagged.indexOf("</PROTEIN>", entIndex + 1);
            int tagEnd = tagged.indexOf(" ", entEnd);

            //get entity from above protein tag locations
            String entity = tagged.substring(entIndex + 1, entEnd - 1);
            //entity = entity.replace("\\(", "-LBR-").replace("\\)", "-RBR-").replace("%", "percent");
            if(entity.contains(",")){
            	LogService.getRoot().log(Level.FINE, entity+" contains a comma");
            }
            //get character offset of this entity
            findIndex = origText.indexOf(entity, findIndex + 1);
            tagged = tagged.substring(tagEnd, tagged.length());
            entID++;
            Protein protein = new Protein();
            protein.index = entID;
            protein.name = entity;
            protein.start_offset = findIndex;
            protein.end_offset = findIndex + entity.length();
            origSentence.addProtein(protein);
            
            findIndex += entity.length();
        }
        
    }



}
