package deeper.operators;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeString;

import deeper.entities.Abstract;
import deeper.entities.rm.io.TextDataIO;

public class PubMedReader extends Operator {

	private static String utils = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/";

	public static String PARAMETER_BEGIN_DATE = "begin_date";
	public static String PARAMETER_END_DATE = "end_date";

	// TODO define metadata
	OutputPort textsOutput = getOutputPorts().createPort("texts");
	//OutputPort sentencesOutput = getOutputPorts().createPort("sentences");
	//OutputPort innerTextSource = getSubprocess(0).getInnerSources().createPort("text");
	// InputPort innerTextSink =
	// getSubprocess(0).getInnerSinks().createPort("text", AbstractIO.class);
	//InputPort innerSentencesSink = getSubprocess(0).getInnerSinks().createPort("sentences", SentencesIO.class);

	public PubMedReader(OperatorDescription description) {
		super(description);
	}

	public void doWork() throws OperatorException {
		//File file = downloadTexts();
		//File file = getProcess().resolveFileName("querylog" + File.separator + "pubmedOut2.xml");
		//File file = new File("querylog" + File.separator + "test.xml");
		File file = new File("c:\\eclipsews\\Pipelines\\querylog\\cellCycle.xml");
		List<Abstract> result = readAbstracts(file);
		TextDataIO<Abstract> resAbstracts = new TextDataIO<Abstract>(result);
		textsOutput.deliver(resAbstracts);
	}

	// method that sends a query to PubMed and gets result. Taken from the old
	// source
	// returns a file where the data is written
	private File downloadTexts() throws OperatorException {
		File resultFile = null;
		// query per day
		String queryLine = utils.concat("esearch.fcgi?db=").concat("pubmed").concat("&mindate=").concat(getParameterAsString(PARAMETER_BEGIN_DATE)).// concat("2010/03/30").
				concat("&maxdate=").concat(getParameterAsString(PARAMETER_END_DATE)).// concat("2010/03/31").
				concat("&retmax=1&usehistory=y");
		// "&term=").concat(query);// + "&retmax=100";
		// queryLine = "http://www.ncbi.nlm.nih.gov/pubmed?term=insulin";
		// medline[sb]+AND+english[la]+AND+hasabstract+AND+
		try {
			URL url = new URL(queryLine);
			HttpURLConnection connect = (HttpURLConnection) url.openConnection();
			BufferedReader in = new BufferedReader(new InputStreamReader(connect.getInputStream(), "UTF-8"));
			String inputLine;
			StringBuffer content = new StringBuffer("");
			while ((inputLine = in.readLine()) != null) {
				content.append(inputLine);
			}
			File dir = getProcess().resolveFileName("querylog");
			if (!dir.exists()) {
				dir.mkdirs();
			}
			PrintWriter queryOut = new PrintWriter(new FileWriter(dir.getAbsolutePath() + File.separator + "pubmedOut1.xml"));
			queryOut.println(content);
			queryOut.flush();
			queryOut.close();
			// System.out.println(content);
			Pattern p = Pattern.compile("<Count>(\\d+)</Count>.*<QueryKey>(\\d+)</QueryKey>.*<WebEnv>(\\S+)</WebEnv>");
			Matcher m = p.matcher(content.toString());
			m.find();
			String count = m.group(1);
			String queryKey = m.group(2);
			String webEnv = m.group(3);
			// RETURN MAX TRY TO LIMIT TO 50
			// int retMax = 20;
			// if (Integer.parseInt(count) > 20000) {
			// ((org.apache.commons.logging.Log)
			// _log).info(count.concat(" abstracts were received which is very strange. I wouldn't save any of them."));
			// count = "10000";
			// }
			// change i < countInt to i < 1, to force one iteration of 50
			// abstracts
			// for(int i = 0, countInt = Integer.parseInt(count);i<1;i+=retMax){
			// String efetch =
			// utils.concat("efetch.fcgi?rettype=").concat(report).
			String efetch = utils.concat("efetch.fcgi?").concat("retmode=xml&retstart=0").concat("&retmax=").concat(count).concat("&db=").concat("pubmed").concat("&query_key=").concat(queryKey)
					.concat("&WebEnv=").concat(webEnv);
			url = new URL(efetch);
			connect = (HttpURLConnection) url.openConnection();
			in = new BufferedReader(new InputStreamReader(connect.getInputStream(), "UTF-8"));
			content = new StringBuffer("");
			inputLine = "";
			while ((inputLine = in.readLine()) != null) {
				content.append(inputLine);
			}
			resultFile = new File(dir.getAbsolutePath() + File.separator + "pubmedOut2.xml");
			PrintWriter out = new PrintWriter(new FileWriter(resultFile));
			out.println(content);
			out.println();
			out.flush();
			out.close();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return resultFile;

	}

	public List<Abstract> readAbstracts(File file) throws OperatorException {
		/*XMLParser handler = new XMLParser();
		SAXParserFactory factory = SAXParserFactory.newInstance();
		try{
			SAXParser saxParser = factory.newSAXParser();
			saxParser.parse(new FileInputStream(file), handler);
		} catch (Throwable t) {
			t.printStackTrace();
		}
		return handler.result;
		*/
		StringBuffer content = new StringBuffer("");
		try {
			BufferedReader reader = new BufferedReader(new FileReader(file));
			String s;
			while ((s = reader.readLine()) != null) {
				content.append(s);
			}
		} catch (IOException e) {
			throw new OperatorException("Cannot read pubmed result file", e);
		}
		List<Abstract> result = new ArrayList<Abstract>();
		Pattern p = Pattern.compile("</PubmedArticle>\\s*<PubmedArticle>");
		String[] abstracts = p.split(content.toString());
		Pattern p_text = Pattern.compile("<AbstractText>([^<]+)</AbstractText>");
		// Trying to catch PubMed ID's -
		Pattern p_id = Pattern.compile("<PMID>(\\d*)</PMID>");
		Pattern p_title = Pattern.compile("<ArticleTitle>([^<]+)</ArticleTitle>");
		Pattern p_date = Pattern.compile("<DateCreated>\\s*<Year>\\s*(\\d+)\\s*</Year>\\s*<Month>\\s*(\\w+)\\s*</Month>\\s*<Day>\\s*(\\d+)\\s*</Day>\\s*</DateCreated>");
		Pattern p_journal = Pattern.compile("<Title>([^<]+)</Title>\\s*</Journal>");
		Matcher m;
		for (int j = 0, countJ = abstracts.length; j < countJ; j++) {
			String text = "", id = "", title = "", journal = "";
			Date date = null;
			m = p_text.matcher(abstracts[j]);
			if (m.find()) {
				text = m.group(1);
			}
			m = p_id.matcher(abstracts[j]);
			if (m.find()) {
				id = m.group(1);
			}
			m = p_title.matcher(abstracts[j]);
			if (m.find()) {
				title = m.group(1);
			}
			m = p_date.matcher(abstracts[j]);
			if (m.find()) {
				DateFormat format = new SimpleDateFormat("yyyy-MM-dd");
				try {
					date = format.parse(m.group(1) + "-" + m.group(2) + "-" + m.group(3));
				} catch (ParseException e) {
					date = new Date(0);
				}
			}
			m = p_journal.matcher(abstracts[j]);
			if(m.find()){
				journal = m.group(1);
			}
			// TODO add author parsing
			Abstract abst = new Abstract();
			abst.setExt_id(id);
			abst.corpus = "pubmed";
			abst.setTitle(title);
			abst.text = text;
			abst.dateWritten = date;
			abst.journalTitle = journal;
			result.add(abst);
		}
		return result;
	}

	@Override
	public List<ParameterType> getParameterTypes() {
		List<ParameterType> types = super.getParameterTypes();
		ParameterType type = new ParameterTypeString(PARAMETER_BEGIN_DATE, "Lower date bound for query", false);
		types.add(type);
		type = new ParameterTypeString(PARAMETER_END_DATE, "Upper date bound for query", false);
		types.add(type);
		return types;
	}

}

