package deeper.operators;

import java.io.IOException;
import java.util.List;

import opennlp.tools.lang.english.SentenceDetector;
import opennlp.tools.sentdetect.SentenceDetectorME;

import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeFile;

import deeper.entities.Abstract;
import deeper.entities.Sentence;
import deeper.entities.rm.io.AbstractIO;

public class SentenceSplitterApplier extends Operator {

	public static String PARAMETER_SENT_SPLIT_MODEL = "sentence_split_model";

	InputPort textInput = getInputPorts().createPort("text", AbstractIO.class);
	OutputPort textOutput = getOutputPorts().createPort("text");

	SentenceDetectorME splitter = null;

	public SentenceSplitterApplier(OperatorDescription description) {
		super(description);
		// TODO pass texts through from textsInput to textsOutput and remove
		// corresponding 'deliver' line from doWork()
	}

	public void doWork() throws OperatorException {
		AbstractIO abstIO = textInput.getData();
		Abstract abst = abstIO.getAbstract();

		try {
			if (splitter == null)
				splitter = new SentenceDetector(getProcess().resolveFileName(getParameterAsString(PARAMETER_SENT_SPLIT_MODEL)).getAbsolutePath());
		} catch (IOException e) {
			throw new OperatorException("Cannot initialize sentence detector", e);
		}
		String[] sentArr = splitter.sentDetect(abst.text);
		for (String text : sentArr) {
			Sentence sentence = new Sentence(text);
			abst.addSentence(sentence);
		}

		textOutput.deliver(abstIO);
	}

	@Override
	public List<ParameterType> getParameterTypes() {
		List<ParameterType> types = super.getParameterTypes();
		ParameterType type;
		type = new ParameterTypeFile(PARAMETER_SENT_SPLIT_MODEL, "A model that is used to split sentences", "", false);
		types.add(type);
		return types;
	}
}
