package deeper.operators;

import java.util.List;

import com.rapidminer.operator.OperatorChain;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeCategory;

import deeper.entities.Abstract;
import deeper.entities.Interaction;
import deeper.entities.Sentence;
import deeper.entities.rm.io.AbstractIO;
import deeper.entities.rm.io.InteractionIO;
import deeper.entities.rm.io.SentenceIO;
import deeper.entities.rm.io.TextDataIO;

/**
 * A wrapper opertor that takes a collection of abstracts and feed the abstracts from this collection
 * one by one to internal operator(s). This aws implemented because for some operators 
 * (such as sentence splitter or parser) it is more natural to hav a single abstract as input.
 * @author tfairuzo
 *
 */
public class TextWrapperOperator extends OperatorChain{
	
	public static String PARAMETER_GRANULARITY = "granulatity";
	
	public static final String[] GRANULARITY_LEVELS = { "abstract" , "sentence", "interaction" };
	
	public static final int GRANULARITY_LEVEL_ABSTRACT = 0;
	
	public static final int GRANULARITY_LEVEL_SENTENCE = 1;
	
	public static final int GRANULARITY_LEVEL_INTERACTION = 2;
	
	InputPort textsIn = getInputPorts().createPort("texts", TextDataIO.class);
	//InputPort exampleSetIn = getInputPorts().createPort("example set", ExampleSet.class);
	OutputPort textsOut = getOutputPorts().createPort("texts");
	
	OutputPort innerTextSource = getSubprocess(0).getInnerSources().createPort("text");
	InputPort innerTextSink = getSubprocess(0).getInnerSinks().createPort("text");
	
	public TextWrapperOperator(OperatorDescription desc){
		super(desc, "Inner Container");
	}
	
	public void doWork() throws OperatorException{
		TextDataIO<Abstract> texts = textsIn.getData();
		int gran = getParameterAsInt(PARAMETER_GRANULARITY);
		//copying data to the result
		TextDataIO<Abstract> result  = new TextDataIO<Abstract>();
		for (Abstract abst : texts.getData()) {
			if(gran==GRANULARITY_LEVEL_ABSTRACT){
				AbstractIO abstIO = new AbstractIO(abst);
				innerTextSource.deliver(abstIO);
				// execute inner operator per abstract
				getSubprocess(0).execute();
				AbstractIO processed = innerTextSink.getData();
				abst = processed.getAbstract();
			}else{
				for(Sentence s: abst.getSentences()){
					if(gran==GRANULARITY_LEVEL_SENTENCE){
						SentenceIO sIO = new SentenceIO(s);
						innerTextSource.deliver(sIO);
						// execute inner operator per sentense
						getSubprocess(0).execute();
						SentenceIO processed = innerTextSink.getData();
						//alter the sentence only if a different object is returned
						if(s!=processed.getSentence()){
							abst.addSentence(processed.getSentence());
						}
					}else{
						for(Interaction i : s.getInteractions()){
							InteractionIO iIO = new InteractionIO(i);
							innerTextSource.deliver(iIO);
							// execute inner operator per interaction
							getSubprocess(0).execute();
							InteractionIO processed = innerTextSink.getData();
							//alter the interaction only if a different object is returned
							if(i!=processed.getInteraction()){
								s.addInteraction(processed.getInteraction());
							}
						}
					}
				}
			}
			result.addElement(abst);
		}
		textsOut.deliver(result);
	}
	
    @Override
    public List<ParameterType> getParameterTypes() {
        List<ParameterType> types = super.getParameterTypes();
        types.add(new ParameterTypeCategory(PARAMETER_GRANULARITY, "Defines the level of detail delivered to the inner operator", GRANULARITY_LEVELS, GRANULARITY_LEVEL_ABSTRACT));
        return types;
    }

}
