package deeper.operators;

import java.util.List;

import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeBoolean;
import com.rapidminer.parameter.ParameterTypeCategory;
import com.rapidminer.parameter.ParameterTypeList;
import com.rapidminer.tools.Ontology;
import com.rapidminer.tools.jdbc.DatabaseHandler;

import deeper.entities.Interaction;
import deeper.entities.enums.InteractionLabel;
import deeper.entities.rm.io.InteractionIO;

/**
 * Operator that extracts the features used in treeSVM
 * @author tfairuzo
 *
 */
public class TreeVectorCompilerOperator extends Operator{

	public static String PARAMETER_MODE = "compiler_mode";
	public static String PARAMETER_SKIP_NULLS = "slip_nulls";
	public static String[] COMPILER_MODES = new String[]{"training","evaluation","prediction"};
	public static int MODE_TRAINING = 0; 
	public static int MODE_EVALUATION = 1;
	public static int MODE_PREDICTION = 2;
	
	InputPort interactionInput = getInputPorts().createPort("interaction", InteractionIO.class);
	OutputPort interactionOutput = getOutputPorts().createPort("interaction");

	public TreeVectorCompilerOperator(OperatorDescription description) {
		super(description);
	}
	
	public void doWork() throws OperatorException{
		InteractionIO interIO = interactionInput.getData();
		Interaction inter = interIO.getInteraction();
		if(getParameterAsBoolean(PARAMETER_SKIP_NULLS)&&
				(inter.getParsePath()==null||inter.getDepPath().getVertices().size()==0)){
			interactionOutput.deliver(null);
			return;
		}
		interIO.setValue("id", inter.getExt_id(), Ontology.NOMINAL);
		interIO.setValue("depTree", inter.getDepPath(), Ontology.NOMINAL);
		interIO.setValue("parseTree", inter.getParsePath(), Ontology.NOMINAL);
		if(inter.getDepPath()==null){
			System.out.println(inter.getExt_id());
		}
		String mode = getParameterAsString(PARAMETER_MODE); 
		if(mode.equals(COMPILER_MODES[MODE_TRAINING])){
			interIO.setValue("label", inter.label.toString(), Ontology.POLYNOMINAL);
		} else if(mode.equals(COMPILER_MODES[MODE_EVALUATION])||mode.equals(COMPILER_MODES[MODE_PREDICTION])){
			interIO.setValue("label", inter.label.toString(), Ontology.POLYNOMINAL);
			if(interIO.getInteraction().prediction!=null){
				interIO.setValue("prediction", interIO.getInteraction().prediction.toString(), Ontology.POLYNOMINAL);
				interIO.setValue("confidence", interIO.getInteraction().confidence, Ontology.NUMERICAL);
				interIO.setValue("probability", interIO.getInteraction().probability, Ontology.NUMERICAL);
			}else{
				//TODO: thorough check whether it's legal
				interIO.setValue("prediction", InteractionLabel.UNDECIDED.toString(), Ontology.POLYNOMINAL);
				interIO.setValue("confidence", -1.0, Ontology.NUMERICAL);
				interIO.setValue("probability", -1.0, Ontology.NUMERICAL);
			}
		} //else if(mode.equals(COMPILER_MODES[MODE_PREDICTION])){
		//}
		//if(interIO.getLabel()!=null){
		//if(interIO.getInteraction().confidence>=0){
		//	interIO.setValue("label", interIO.getLabel().toString(), Ontology.POLYNOMINAL);
		//}
		interactionOutput.deliver(interIO);
	}
	
	@Override
	public List<ParameterType> getParameterTypes() {
		//add parameters from superclass
		List<ParameterType> types = super.getParameterTypes();
		
		ParameterType type = new ParameterTypeCategory(PARAMETER_MODE,"Which special attributes should be included in the vector: label, label and prediction or nothing",COMPILER_MODES,MODE_PREDICTION);
		types.add(type);
		
		type = new ParameterTypeBoolean(PARAMETER_SKIP_NULLS,"Skip instances that have no or empty parse tree",true);
		types.add(type);
		
		return types;
	}

}
