package deeper.operators.converters;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.AttributeRole;
import com.rapidminer.example.Attributes;
import com.rapidminer.example.Example;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.example.table.AttributeFactory;
import com.rapidminer.example.table.ExampleTable;
import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.UserError;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeBoolean;
import com.rapidminer.parameter.ParameterTypeCategory;
import com.rapidminer.parameter.ParameterTypeDouble;
import com.rapidminer.parameter.ParameterTypeFile;
import com.rapidminer.parameter.ParameterTypeInt;
import com.rapidminer.tools.Ontology;

import deeper.entities.Abstract;
import deeper.entities.Interaction;
import deeper.entities.Sentence;
import deeper.entities.enums.InteractionLabel;
import deeper.entities.rm.io.TextDataIO;

public class TreeSVMApplier extends Operator{
	
	public static String PARAMETER_TEST_FILE = "test_file";
	public static String PARAMETER_OUT_FILE = "out_file";
	public static String PARAMETER_TEST = "test_exec";
	public static String PARAMETER_RESULT_FILE = "result_file";
	public static String PARAMETER_PROB_FILE = "prob_file";
	//public static String PARAMETER_MODEL = "model";
	//public static String PARAMETER_A = "a";
	//public static String PARAMETER_B = "b";
	public static String PARAMETER_VECTOR_TYPE = "vector_type";
	public static String[] VECTOR_TYPES= new String[]{"lex","pos","syn","lex+pos","lex+syn","lex+pos+syn","pos+syn"};
	public static int LEX = 0;
	public static int POS = 1;
	public static int SYN = 2;
	public static int LEX_POS = 3;
	public static int LEX_SYN = 4;
	public static int LEX_POS_SYN = 5;
	public static int POS_SYN = 6;
	//public static String PARAMETER_C_VALUE = "c_value";
	//public static String PARAMETER_L_VALUE = "L_value";
	
	InputPort inputTexts = getInputPorts().createPort("texts", TextDataIO.class);
	InputPort inputExampleSet = getInputPorts().createPort("example set", ExampleSet.class);
	InputPort modelInput = getInputPorts().createPort("model", TreeSVMModel.class);
	OutputPort outputTexts = getOutputPorts().createPort("texts");
	OutputPort outputExampleSet = getOutputPorts().createPort("example set");

	public TreeSVMApplier(OperatorDescription description) {
		super(description);
	}
	
	public void doWork() throws OperatorException{
		TextDataIO<Abstract> absts = inputTexts.getDataOrNull();
		ExampleSet exSet = inputExampleSet.getDataOrNull();
		TreeSVMModel model = modelInput.getData();
		if(absts!=null&&exSet!=null){
			throw new UserError(this,127,"Either texts or ExampleSet should be provided to the svm applier, but not both");
		}
		InteractionLabel[] labels = null;
		double[] predictions = null;
		String svm_classify = getProcess().resolveFileName(getParameterAsString(PARAMETER_TEST)).getAbsolutePath();
		String testFile = getProcess().resolveFileName(getParameterAsString(PARAMETER_TEST_FILE)).getAbsolutePath();
		String predictionsFile = getProcess().resolveFileName(getParameterAsString(PARAMETER_OUT_FILE)).getAbsolutePath();
		try{			
			if(absts!=null){
				labels = SVMLightConverters.abstracts2Moschitti(absts.getData(), testFile, getParameterAsInt(PARAMETER_VECTOR_TYPE));
			}else{
				labels = SVMLightConverters.abstracts2Moschitti(exSet, testFile, getParameterAsInt(PARAMETER_VECTOR_TYPE));
			}
		} catch (IOException e) {
			throw new OperatorException("Can't write the test file "+testFile ,e);
		}
		SVMLight svm = new SVMLight("",svm_classify,model.modelFile);//getProcess().resolveFileName(getParameterAsString(PARAMETER_MODEL)).getAbsolutePath());
		svm.test(testFile,predictionsFile);
		try {
			if(absts!=null){
				predictions = SVMLightConverters.predictions2Abstracts(absts.getData(), predictionsFile);
			}else{
				predictions = SVMLightConverters.predictions2ExampleSet(exSet, predictionsFile);
			}
		} catch (IOException e) {
			throw new OperatorException("Can't read predictions file "+predictionsFile,e);
		}
		if(getParameters().isSet(PARAMETER_RESULT_FILE)){
			File file = getProcess().resolveFileName(getParameterAsString(PARAMETER_RESULT_FILE)).getAbsoluteFile();
			try {
				FileWriter writer = new FileWriter(file,true);
				for(int i = 0; i<labels.length;i++){
					//do not take undecided into account
					String label  ="";
					if(labels[i]==InteractionLabel.POSITIVE){
						label = "true";
					}else{
						label = "false";
					}

					writer.write(predictions[i]+"	"+label);
					writer.write("\n");
				}
				writer.close();
			} catch (IOException e) {
				throw new OperatorException("Cannot write to the predictions file "+file.getAbsolutePath(),e);
			}
		}
		if(model.a!=0 || model.b!=0){
			PlattProbabilityEstimator estimator = new PlattProbabilityEstimator(model.a,model.b);
			if(absts!=null){
				for (Abstract abst : absts.getData()) {
					for (Sentence s : abst.getSentences()) {
						for (Interaction i : s.getInteractions()) {
							if (i.getDepPath() != null && i.getDepPath().getVertices().size() != 0) {
								i.probability = estimator.getProbability(i.confidence);
							}
						}
					}
				}
			}else{
				//add probability to example set
				Attribute probAtt = AttributeFactory.createAttribute("probability",Ontology.NUMERICAL);
				ExampleTable table = exSet.getExampleTable();
				table.addAttribute(probAtt);
				exSet.getAttributes().setSpecialAttribute(probAtt, "other");
				Iterator<Example> it = exSet.iterator();
				int k = 0;
				while (it.hasNext()) {
					Example ex = it.next();
					ex.setValue(probAtt, estimator.getProbability(predictions[k++]));
				}
			}
		}
		
		outputTexts.deliver(absts);
		outputExampleSet.deliver(exSet);
	}

	//input parameter: model file
	
	//input: example set
	//output: example set with prediction class and confidence
	
	//convert example set to a file
	//run svmLight on the file
	//convert outputFile to an example set (or at least read f-values)
	
	@Override
	public List<ParameterType> getParameterTypes() {
		List<ParameterType> types = super.getParameterTypes();

		ParameterType type;

		type = new ParameterTypeFile(PARAMETER_TEST_FILE, "File with the test data","",true);
		types.add(type);
		
		type = new ParameterTypeFile(PARAMETER_OUT_FILE, "Predictions file","",true);
		types.add(type);
		
		type = new ParameterTypeFile(PARAMETER_PROB_FILE, "Probabilities file","",false);
		types.add(type);
		
		type = new ParameterTypeFile(PARAMETER_TEST, "A path to svm_classify","",true);
		types.add(type);
		
		type = new ParameterTypeFile(PARAMETER_RESULT_FILE, "A file to append predictions (intended to use for probability estimation)","",false);
		types.add(type);
		
		//type = new ParameterTypeFile(PARAMETER_MODEL, "A path to model file","",true);
		//types.add(type);
		
		type = new ParameterTypeCategory(PARAMETER_VECTOR_TYPE, "Defines which trees to include in the feature vectors",VECTOR_TYPES,LEX_POS_SYN);
		types.add(type);
		
		//type = new ParameterTypeDouble(PARAMETER_A, "Parameter for the prediction probability estimation", Double.NEGATIVE_INFINITY, Double.POSITIVE_INFINITY,true);
		//types.add(type);
		
		//type = new ParameterTypeDouble(PARAMETER_B, "Parameter for the prediction probability estimation", Double.NEGATIVE_INFINITY, Double.POSITIVE_INFINITY,true);
		//types.add(type);
		
		/*type = new ParameterTypeDouble(PARAMETER_C_VALUE, "This is the -c arugment for the svm training", -1000, 1000);
		types.add(type);
		
		type = new ParameterTypeDouble(PARAMETER_L_VALUE, "Decay factor for tree kernels", 0, 1000);
		types.add(type);
		*/
		return types;
	}

	
}
