package deeper.operators.converters;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import com.rapidminer.FileProcessLocation;
import com.rapidminer.Process;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.operator.IOContainer;
import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorChain;
import com.rapidminer.operator.OperatorCreationException;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.UserError;
import com.rapidminer.operator.performance.PerformanceEvaluator;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.operator.ports.metadata.PassThroughRule;
import com.rapidminer.operator.validation.XValidation;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeCategory;
import com.rapidminer.parameter.ParameterTypeDouble;
import com.rapidminer.parameter.ParameterTypeFile;
import com.rapidminer.tools.OperatorService;

import deeper.entities.Abstract;
import deeper.entities.rm.io.TextDataIO;

public class TreeSVMLearner extends Operator {

	public static String PARAMETER_FILE = "train_file";
	public static String PARAMETER_TRAIN = "train_exec";
	public static String PARAMETER_TEST = "test_exec";
	public static String PARAMETER_MODEL = "model";
	public static String PARAMETER_AB = "ab_file";
	public static String PARAMETER_ESTIMATOR_DIR = "estimator_dir";
	public static String PARAMETER_C_VALUE = "c_value";
	public static String PARAMETER_L_VALUE = "L_value";
	public static String PARAMETER_VECTOR_TYPE = "vector_type";
	public static String[] VECTOR_TYPES= new String[]{"lex","pos","syn","lex+pos","lex+syn","lex+pos+syn","pos+syn"};
	public static int LEX = 0;
	public static int POS = 1;
	public static int SYN = 2;
	public static int LEX_POS = 3;
	public static int LEX_SYN = 4;
	public static int LEX_POS_SYN = 5;
	public static int POS_SYN = 6;
	
	InputPort inputTexts = getInputPorts().createPort("texts", TextDataIO.class);
	InputPort inputExampleSet = getInputPorts().createPort("example set", ExampleSet.class);
	OutputPort outputTexts = getOutputPorts().createPort("texts through");
	OutputPort outputExampleSet = getOutputPorts().createPort("example set through");
	OutputPort model = getOutputPorts().createPort("model");
	
	public TreeSVMLearner(OperatorDescription description) {
		super(description);
		getTransformer().addRule(new PassThroughRule(inputTexts, outputTexts, false));
		getTransformer().addRule(new PassThroughRule(inputExampleSet, outputExampleSet, false));
	}

	//parameter: model file
	
	//input example set
	
	//convert example set to  file
	//train svm
	
	public void doWork() throws OperatorException{
		TextDataIO<Abstract> absts = inputTexts.getDataOrNull();
		ExampleSet exSet = inputExampleSet.getDataOrNull();
		if(absts!=null&&exSet!=null){
			throw new UserError(this,127,"Either texts or ExampleSet should be provided to the learner, but not both");
		}
		try{
			if(absts!=null){
				SVMLightConverters.abstracts2Moschitti(absts.getData(), getProcess().resolveFileName(getParameterAsString(PARAMETER_FILE)).getAbsolutePath(), getParameterAsInt(PARAMETER_VECTOR_TYPE));
			}else{
				SVMLightConverters.abstracts2Moschitti(exSet, getProcess().resolveFileName(getParameterAsString(PARAMETER_FILE)).getAbsolutePath(), getParameterAsInt(PARAMETER_VECTOR_TYPE));
			}
		} catch (IOException e) {
			throw new OperatorException("Can't write training file",e);
		}
		//if we want to estimate probabilities, we need to run cross-validation on a training set
		double[] ab = null;
		if(getParameters().isSet(PARAMETER_ESTIMATOR_DIR)){
			if(exSet==null){
				throw new UserError(this,127,"Probability estimation can be done only when ExampleSet is used as an input");
			}
			try {
				ab = estimateProbabilities(getProcess().resolveFileName(getParameterAsString(PARAMETER_ESTIMATOR_DIR)).getAbsolutePath(),
						getProcess().resolveFileName(getParameterAsString(PARAMETER_TRAIN)).getAbsolutePath(),
						getProcess().resolveFileName(getParameterAsString(PARAMETER_TEST)).getAbsolutePath(),
						exSet,
						getParameterAsDouble(PARAMETER_C_VALUE),
						getParameterAsDouble(PARAMETER_L_VALUE), 
						getParameterAsInt(PARAMETER_VECTOR_TYPE));
			} catch (IOException e) {
				throw new OperatorException("Cannot read xvalidation result file",e);
			} catch (OperatorCreationException e) {
				throw new OperatorException("Cannot create sub pipeline",e);
			}
		}
		
		SVMLight svm = new SVMLight(getProcess().resolveFileName(getParameterAsString(PARAMETER_TRAIN)).
				getAbsolutePath(),"",getProcess().resolveFileName(getParameterAsString(PARAMETER_MODEL)).
				getAbsolutePath(), getParameterAsDouble(PARAMETER_C_VALUE),getParameterAsDouble(PARAMETER_L_VALUE));
		svm.train(getProcess().resolveFileName(getParameterAsString(PARAMETER_FILE)).getAbsolutePath());
		
		//RM model to be compliant with RM pipelines
		TreeSVMModel mod = new TreeSVMModel();
		mod.modelFile = getProcess().resolveFileName(getParameterAsString(PARAMETER_MODEL)).getAbsolutePath();
		if(ab!=null){
			mod.a = ab[0];
			mod.b = ab[1];
			try {
				FileWriter abw = new FileWriter(getProcess().resolveFileName(getParameterAsString(PARAMETER_AB)).getAbsoluteFile());
				abw.write(ab[0]+"\n"+ab[1]);
				abw.close();
			} catch (IOException e) {
				throw new OperatorException("Cannot write probability parameter file");
			}
		}
		model.deliver(mod);
	}
	

	private double[] estimateProbabilities(String svmDir, String svm_learn, String svm_classify, ExampleSet train, double c, double L, int vectorType) throws OperatorException, OperatorCreationException, IOException{
		double[] ab = new double[2];
		
		//create a process that performs the internal XValidation on the training set
		Process process = new Process();
		process.setProcessLocation(new FileProcessLocation(new File(svmDir+File.separator+"xval.rmp")));
		OperatorChain xval = OperatorService.createOperator(XValidation.class);
		process.getRootOperator().getSubprocess(0).addOperator(xval);
		Operator svm = OperatorService.createOperator(TreeSVMLearner.class);
		xval.getSubprocess(0).addOperator(svm);
		svm.setParameter(TreeSVMLearner.PARAMETER_TRAIN, svm_learn);
		svm.setParameter(TreeSVMLearner.PARAMETER_TEST, svm_classify);
		svm.setParameter(TreeSVMLearner.PARAMETER_FILE, svmDir+File.separator+"train");
		svm.setParameter(TreeSVMLearner.PARAMETER_MODEL, svmDir+File.separator+"model");
		svm.setParameter(TreeSVMLearner.PARAMETER_C_VALUE, Double.toString(c));
		svm.setParameter(TreeSVMLearner.PARAMETER_L_VALUE, Double.toString(L));
		svm.setParameter(TreeSVMLearner.PARAMETER_VECTOR_TYPE, TreeSVMLearner.VECTOR_TYPES[vectorType]);
		
		xval.getSubprocess(0).getInnerSources().getPortByName("training").
			connectTo(svm.getInputPorts().getPortByName("example set"));
		svm.getOutputPorts().getPortByName("model").
			connectTo(xval.getSubprocess(0).getInnerSinks().getPortByName("model"));
		
		Operator applier = OperatorService.createOperator(TreeSVMApplier.class);
		xval.getSubprocess(1).addOperator(applier);
		applier.setParameter(TreeSVMApplier.PARAMETER_TEST, svm_classify);		
		applier.setParameter(TreeSVMApplier.PARAMETER_TEST_FILE, svmDir+File.separator+"test");
		applier.setParameter(TreeSVMApplier.PARAMETER_OUT_FILE, svmDir+File.separator+"prediction");
		applier.setParameter(TreeSVMApplier.PARAMETER_RESULT_FILE, svmDir+File.separator+"result");
		applier.setParameter(TreeSVMApplier.PARAMETER_VECTOR_TYPE,TreeSVMApplier.VECTOR_TYPES[vectorType]);
		
		//delete file in order to start appending to a clean one
		File file = new File(svmDir+File.separator+"result");
		if(file.exists()){
			file.delete();
		}
		
		Operator evaluator = OperatorService.createOperator(PerformanceEvaluator.class);
		xval.getSubprocess(1).addOperator(evaluator);
		
		xval.getSubprocess(1).getInnerSources().getPortByName("model").
			connectTo(applier.getInputPorts().getPortByName("model"));
		
		xval.getSubprocess(1).getInnerSources().getPortByName("test set").
			connectTo(applier.getInputPorts().getPortByName("example set"));
		
		applier.getOutputPorts().getPortByName("example set").
			connectTo(evaluator.getInputPorts().getPortByName("labelled data"));
		
		evaluator.getOutputPorts().getPortByName("performance").
			connectTo(xval.getSubprocess(1).getInnerSinks().getPortByName("averagable 1"));
		
		//xval.getInputPorts().getPortByName("training").receive(train);
		
		process.getRootOperator().getSubprocess(0).getInnerSources().getPortByName("input 1").
			connectTo(xval.getInputPorts().getPortByName("training"));
		process.run(new IOContainer(train));

		//after successful execution of xvalidation process we should have a predictions file to estimate probability function parameters
		
		List<Double> predictions = new ArrayList<Double>();
		List<Boolean> labels = new ArrayList<Boolean>();
		
		BufferedReader reader = new BufferedReader(new FileReader(svmDir+File.separator+"result"));
		String s= "";
		while((s=reader.readLine())!=null){
			String[] split = s.split("\\s+");
			predictions.add(Double.parseDouble(split[0]));
			labels.add(Boolean.parseBoolean(split[1]));
		}
		
		int i = 0;
		int numPos = 0, numNeg = 0;
		double[] d_predictions = new double[predictions.size()];
		boolean[] b_labels = new boolean[labels.size()];
		//prepare data for the probability estimator
		int k = 0;
		for(Boolean l: labels){
			d_predictions[k] = predictions.get(k);
			b_labels[k] = labels.get(k);
			if(b_labels[k]){
				numPos++;
			}else{
				numNeg++;
			}
			k++;
		}
		PlattProbabilityEstimator estimator = new PlattProbabilityEstimator(d_predictions, b_labels, numPos, numNeg);
		
		ab[0] = estimator.a;
		ab[1] = estimator.b;
		
		return ab;
	}
	
	
	@Override
	public List<ParameterType> getParameterTypes() {
		List<ParameterType> types = super.getParameterTypes();

		ParameterType type;

		type = new ParameterTypeFile(PARAMETER_FILE, "File to save the training data in SVMLight format","",true);
		types.add(type);
		
		type = new ParameterTypeFile(PARAMETER_TRAIN, "A path to svm_train","",true);
		types.add(type);
		
		type = new ParameterTypeFile(PARAMETER_TEST, "A path to svm_classify","",true);
		types.add(type);
		
		type = new ParameterTypeFile(PARAMETER_MODEL, "A path to model file","",true);
		types.add(type);
		
		type = new ParameterTypeDouble(PARAMETER_C_VALUE, "This is the -c arugment for the svm training", -1000, 1000);
		types.add(type);
		
		type = new ParameterTypeDouble(PARAMETER_L_VALUE, "Decay factor for tree kernels", 0, 1000);
		types.add(type);
		
		type = new ParameterTypeCategory(PARAMETER_VECTOR_TYPE, "Defines which trees to include in the feature vectors",VECTOR_TYPES,LEX_POS_SYN);
		types.add(type);
		
		type = new ParameterTypeFile(PARAMETER_ESTIMATOR_DIR, "Dir to write temporary classification results for probability estimation purposes. If not set probabilities are not estimated", "", false);
		types.add(type);
		
		type = new ParameterTypeFile(PARAMETER_AB, "File to write the parameters for probability estimation", "", false);
		types.add(type);
				
		return types;
	}

}
