/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package deeper.operators.datasets;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.rapidminer.tools.LogService;
import com.rapidminer.tools.Ontology;

import deeper.entities.Abstract;
import deeper.entities.Interaction;
import deeper.entities.Protein;
import deeper.entities.Sentence;
import deeper.entities.rm.io.AbstractIO;
import deeper.entities.rm.io.InteractionIO;

/**
 *  Iterator class that traverses the PPI XML representation (Pyysalo paper)
 * @author tfairuzo
 */
public class XMLAbstractIterator implements Iterator<Abstract> {

    private NodeList docList = null;
    private int size = 0;

    private Iterator<Node> docIterator = null;

    public XMLAbstractIterator(File xmlFile) {
        //TODO: figure out what to do with exceptions
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document doc = db.parse(xmlFile);
            doc.getDocumentElement().normalize();
            docList = doc.getElementsByTagName("document");
            size = docList.getLength();
            List<Node> documentNodes = new ArrayList<Node>();
            for(int i = 0; i<size; i++){
            	documentNodes.add(docList.item(i));
            }
            //initInteractionList();
            docIterator = documentNodes.iterator();
        } catch (IOException ex) {
            LogService.getRoot().log(Level.SEVERE, "Error reading xml file "+xmlFile,ex);
        } catch (SAXException ex) {
            LogService.getRoot().log(Level.SEVERE, "Error parsing xml file "+xmlFile,ex);
        } catch (ParserConfigurationException ex) {
            LogService.getRoot().log(Level.SEVERE, "", ex);
        }


    }

    @Override
    public boolean hasNext() {
        return docIterator.hasNext();
    }

    /**
     * Returns next interaction from the interaction list.
     * If current interaction list is empty, then reads the next sentence and all
     * entities that are annotated in the sentence, and then creates a fresh list
     * of interactions built from all entity pairs.
     * @return
     */
    @Override
    public Abstract next() {
    	return buildDocumentFromXML(docIterator.next());
    }

    @Override
    public void remove() {
        //no op
    }
    
    private Abstract buildDocumentFromXML(Node docNode){
    	Abstract doc = new Abstract();
    	String extId = docNode.getAttributes().getNamedItem("id").getNodeValue();
    	doc.setExt_id(extId);
    	NodeList sentenceNodes = docNode.getChildNodes();
    	for(int i = 0; i< sentenceNodes.getLength(); i++){
    		Node sNode = sentenceNodes.item(i);
    		String text = sNode.getAttributes().getNamedItem("text").getNodeValue();
    		Sentence sent = new Sentence(text);
    		doc.addSentence(sent);
    		NodeList mixNodes = sNode.getChildNodes();
    		
    		//map accumulates the interactions annotated in XML
    		Map<List<Protein>,String> validInteractions = new HashMap<List<Protein>,String>();
    		
    		for(int j = 0; j< mixNodes.getLength(); j++){
    			Node n = mixNodes.item(i);
                //parse protein definition
                if (n.getNodeName().equals("entity")) {
                    Protein p = new Protein();
                    NamedNodeMap attMap = n.getAttributes();
                    boolean nameSplit = false;
                    p.setExt_id(n.getAttributes().getNamedItem("id").getNodeValue());
                    String[] offsets = n.getAttributes().getNamedItem("charOffset").getNodeValue().split(",");
                    String[] offset = offsets[0].split("-");
                    p.start_offset = Integer.parseInt(offset[0]);
                    offset = offsets[offsets.length - 1].split("-");
                    p.end_offset = Integer.parseInt(offset[1])+1;
                    
                    //if there is more than one offset (i.e. a name is split), we take the whole span as a name
                    //otherwise, the span will coincide with the value of the `name' attribute
                    p.name = text.substring(p.start_offset,p.end_offset);
                                       
                    boolean doNotAdd = false;
                    //if protein overlaps with another one, do not add it
                    for(Protein pp: sent.getProteins()){
                    	if((p.start_offset>=pp.start_offset&&p.start_offset<=pp.end_offset)||
                    			(p.end_offset>=pp.start_offset&&p.end_offset<=pp.end_offset)||
                    			(p.start_offset<=pp.start_offset&&p.end_offset>=pp.end_offset)){
                    		doNotAdd = true;
                    		break;
                    	}
                    }
                    if(!doNotAdd){
                    	/*Integer existProtId = proteinIds.get(p.name);
                    	if(existProtId==null){
                    		proteinIds.put(p.name, protId);
                    		existProtId = protId++;
                    	}
                    	p.id = existProtId;
                    	*/
                    	sent.addProtein(p);
                    }
                }
                //keep the protein indices of annotated interactions
                if (n.getNodeName().equals("interaction")) {
                	Protein p1 = sent.getProteins().getByXmlId(n.getAttributes().getNamedItem("e1").getNodeValue());
                	Protein p2 = sent.getProteins().getByXmlId(n.getAttributes().getNamedItem("e2").getNodeValue());
                	if(p1!=null&&p2!=null){
                		List<Protein> pair = new ArrayList<Protein>();
                		pair.add(p1);
                		pair.add(p2);
                		validInteractions.put(pair,n.getAttributes().getNamedItem("id").getNodeValue());
                	}
                }    			
    		}
    		sent.rewrite();
    		sent.buildInteractions(validInteractions, true, "xml");
    	}
    	return doc;
    }
}
