package deeper.parser;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import edu.stanford.nlp.ling.Sentence;
import edu.stanford.nlp.ling.TaggedWord;

/**
 * Word count starts from 0.
 * 
 * Represents enriched sentence structure.
 * Main reason why I need this class is because I want to have 
 * direct mappings from nodes in dependency and parse trees to
 * words in the sentence. It is not always obvious, since different 
 * parsers (and probably different settings of one parser) could 
 * provide different word indexing (e.g. with/without punctuation or
 * splitting hyphenated words).
 * 
 * Moreover, other information about specific words (such as 
 * interactions, that the word takes part in) could be accumulated 
 * here. WRONG! IT SHOULDN"T BE HERE.
 * 
 *  I don't want to provide an abstract implementation for different 
 *  parsers because the class is quite simple. To add support for a
 *  specific parser one should add constructor that takes the specific 
 *  parser output as an argument.
 * @author tfayruzo
 *
 */
public class StructuredSentence implements Serializable {

    /**
	 * 
	 */
	private static final long serialVersionUID = -5372996546825702830L;
	
	//array as the sentence has unmodifiable length and random access to words
    //is expected
    private Word[] sentence;

    /**
     * Constructor for StanfordParser implementation.
     * @param sentence
     */
    public StructuredSentence(Sentence<TaggedWord> sentence) {
        this.sentence = new Word[sentence.length()];
        int i = 0;
        for (TaggedWord word : sentence) {
            Word w = new Word();
            w.word = word.word();
            w.addFeature("pos", word.tag());
            this.sentence[i++] = w;
        }
    }

    public String getWord(int i) {
        if (i >= 1000) {
            i -= 1000;
        }
        return sentence[i].word;
    }

    public String getPos(int i) {
        if (i >= 1000) {
            i -= 1000;
        }
        return (String) sentence[i].getFeature("pos");
    }

    /**
     * Returns the index of the word in the sentence.
     * If the word occurs more than once, returns the
     * index of the first occurence.
     * @param word
     * @return word index or -1 if the word is not found
     */
    public int getIndex(String word) {
        int i = 0;
        for (Word w : sentence) {
            if (w.word.contains(word)) {
                return i;
            }
            i++;
        }
        return -1;
    }

    @Override
    public String toString(){
        StringBuffer result = new StringBuffer("");
        for(int i=0, length = sentence.length; i<length;i++){
            result.append(sentence[i].word+"-"+i+" ");
        }
        return result.toString();
    }

    class Word implements Serializable {
        /**
		 * 
		 */
		private static final long serialVersionUID = 2161189441848495197L;
		
		//word itself
        public String word;
        //various word features
        private Map<String, Object> features = new HashMap<String, Object>();

        public void addFeature(String key, Object feature){
            features.put(key, feature);
        }

        public Object getFeature(String key){
            return features.get(key);
        }
    }
}
